'''Column of Section.

In most cases, one section per page. But in case multi-columns page, sections are used
to distinguish these different layouts.

.. note::
    Currently, support at most two columns.

::

    {
        'bbox': (x0, y0, x1, y1),
        'blocks': [{
            ... # block instances
        }, ...],
        'shapes': [{
            ... # shape instances
        }, ...]
    }
'''

from ..common.Collection import Collection
from ..layout.Layout import Layout
from ..shape.Shape import Shape
from ..text.Line import Line


class Column(Layout):
    '''Column of Section.'''

    @property
    def working_bbox(self): return self.bbox


    def add_elements(self, elements:Collection):
        '''Add candidate elements, i.e. lines or shapes, to current column.'''
        blocks = [e for e in elements if isinstance(e, Line)]
        shapes = [e for e in elements if isinstance(e, Shape)]
        self.assign_blocks(blocks)
        self.assign_shapes(shapes)


    def make_docx(self, doc):
        '''Create Section Column in docx.

        Args:
            doc (Document): ``python-docx`` document object
        '''
        self.blocks.make_docx(doc)



'''Main window for ``pdf2docx`` graphic user interface.'''

import sys
from tkinter import Tk, messagebox
from .MainFrame import MainFrame


class App(Tk):
    '''Simple graphic user interface.'''
    def __init__(self, title:str='App', width:int=300, height:int=200):
        '''Top app window.'''
        super().__init__()
        self.title(title)
        self.geometry(f'{width}x{height}')
        self.resizable(0, 0) # not allowed to change size

        # layout on the root window
        self.__create_widgets()

        # Quit window
        self.protocol("WM_DELETE_WINDOW", self._on_closing)


    def __create_widgets(self):
        self.widget = MainFrame(self)
        self.widget.grid(column=0, row=0)


    def _on_closing(self):
        if messagebox.askokcancel("Quit", "Do you want to quit?"):
            self.destroy()
            sys.exit(0)


if __name__ == "__main__":
    app = App(title='PDF_2_Docx Converter', width=500, height=600)
    app.mainloop()



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath("../pdf2docx/"))


# -- Project information -----------------------------------------------------

project = 'pdf2docx'
copyright = '2023, Artifex'
author = 'Artifex Software, Inc.'

# The full version, including alpha/beta/rc tags
# read version number from version.txt, otherwise alpha version
# Github CI can create version.txt dynamically.
def get_version(fname):
    if os.path.exists(fname):
        with open(fname, 'r') as f:
            version = f.readline().strip()
    else:
        version = 'alpha'

    return version
release = get_version('../version.txt')

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinxcontrib.apidoc'
]

apidoc_module_dir = '../pdf2docx'
apidoc_output_dir = 'api'
apidoc_excluded_paths = []
apidoc_separate_modules = True

# Add any paths that contain templates here, relative to this directory.
# templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [    
]


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = 'sphinx_rtd_theme'


# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# -- Options for LaTeX output ---------------------------------------------
latex_elements = {
    # "fontpkg": r"\usepackage[sfdefault]{ClearSans} \usepackage[T1]{fontenc}"
}
# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [("index", "pdf2docx.tex", "pdf2docx Documentation", "Artifex", "manual")]
# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = "images/pymupdf-logo.png"

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
# latex_use_parts = False

# If true, show page references after internal links.
latex_show_pagerefs = False

# If true, show URL addresses after external links.
# latex_show_urls = True
# latex_use_xindy = True
# Documents to append as an appendix to all manuals.
# latex_appendices = []

# If false, no module index is generated.
latex_domain_indices = True

# -- Options for PDF output --------------------------------------------------
# Grouping the document tree into PDF files. List of tuples
# (source start file, target name, title, author).

pdf_documents = [("index", "pdf2docx", "pdf2docx manual", "Artifex")]

# A comma-separated list of custom stylesheets. Example:
# pdf_stylesheets = ["sphinx", "bahnschrift", "a4"]

# Create a compressed PDF
pdf_compressed = True

# A colon-separated list of folders to search for fonts. Example:
# pdf_font_path=['/usr/share/fonts', '/usr/share/texmf-dist/fonts/']

# Language to be used for hyphenation support
pdf_language = "en_US"

# If false, no index is generated.
pdf_use_index = True

# If false, no modindex is generated.
pdf_use_modindex = True

# If false, no coverpage is generated.
pdf_use_coverpage = True

pdf_break_level = 2

pdf_verbosity = 0
pdf_invariant = True






'''Main frame with grouped controls.'''

import os
from tkinter import Tk, Frame, Label, Entry, Button, filedialog, messagebox
import webbrowser
from pdf2docx import Converter


class MainFrame(Frame):
    def __init__(self, parent:Tk=None):
        '''Main frame with grouped controls. This should be attached to a parent ``Tk`` instance.'''
        super().__init__(parent)
        self.parent = parent
        self.config(bg='grey')
        self.grid(ipadx=100, ipady=100)
        self.setup_ui()

        # variables
        self.pdf_paths = set() # unique pdf files
        self.docx_folder = None

    
    def setup_ui(self):
        '''Layout of the user interface.'''
        # PDF to Docx Label
        self.program_use_label = Label(self, text='PDF to Docx', font='Impact 40', bg='white', fg='#1E90FF')
        self.program_use_label.grid(row=0, column=0, columnspan=2, padx=120, pady=50)

        # PDF file entry
        self.file_path_pdf_entry = Entry(self, border=5)
        self.file_path_pdf_entry.grid(row=1, column=0, sticky='w', ipady=4, padx=10)

        # Docx file entry
        self.file_path_docx_entry = Entry(self, border=5)
        self.file_path_docx_entry.grid(row=2, column=0, sticky='w', ipady=4, padx=10, pady=75)

        # Select PDF files Button
        self.select_pdf_file = Button(self, text='Select PDF files', 
            fg='black', 
            bg='white', 
            border=3,
            command=self._callback_pdf_file_location)
        self.select_pdf_file.grid(row=1, column=1, sticky='w')

        # Select new files folder Button
        self.select_new_file_folder = Button(self, text='Select new files folder', 
            fg='black', 
            bg='white', 
            border=3,
            command=self._callback_docx_folder_location)
        self.select_new_file_folder.grid(row=2, column=1, sticky='w')

        # Convert Button
        self.converter_button = Button(self, text='Convert', 
            bg='#1E90FF', 
            fg='white', 
            font='impact 20', 
            border=5,
            command=self._callback_convert)
        self.converter_button.grid(row=3, column=0, columnspan=2, ipady=5, ipadx=10)

        # *This converter can only convert text based pdf cant convert image based pdf Label
        self.check_label = Label(self, text='*This converter can only convert text based pdf \n cant convert image based pdf',
                            bg='grey')
        self.check_label.grid(row=4, column=0, columnspan=2, pady=10)

        # how to check my file Button
        self.file_check_button = Button(self, text='how to check my file?', 
            bg='grey', 
            fg='black',
            command=lambda: webbrowser.open_new(r"https://support.policystat.com/hc/en-us/articles"
                        r"/207993346-How-can-I-tell-if-my-PDFs-are-text-based-or-not-"))
        self.file_check_button.grid(row=5, column=0, columnspan=2)


    def _callback_pdf_file_location(self):
        '''Opens file explorer and let you select choose the pdf file that you want to convert.'''
        file_paths = filedialog.askopenfilenames(filetypes=[('PDF file', '*.pdf')])
        self.pdf_paths = set()
        for path in file_paths:
            self.pdf_paths.add(path)
        
        # show just names
        names = ';'.join([f'"{os.path.basename(path)}"' for path in self.pdf_paths])
        self.file_path_pdf_entry.delete(0, 'end')
        self.file_path_pdf_entry.insert(0, names)


    def _callback_docx_folder_location(self):
        '''Opens file explorer and let you choose the folder, that all the converted file or files going to saved.'''
        self.docx_folder = filedialog.askdirectory()
        self.file_path_docx_entry.delete(0, 'end')
        self.file_path_docx_entry.insert(0, self.docx_folder)
    

    def _callback_convert(self):
        '''Starts the convert of the file or files.'''
        # input check
        if not self.pdf_paths and not self.docx_folder:
            messagebox.showwarning(
                title='Neither files or folder selected', 
                message='Select PDF file or files for convert '
                        'and Select a folder for the converted files!')
            return

        if not self.pdf_paths:
            messagebox.showwarning(
                title='Not files for convert selected', 
                message='Select PDF file or PDF files for convert!')
            return

        if not self.docx_folder:
            messagebox.showwarning(
                title='Not files folder selected', 
                message='Select a folder for the converted files!')
            return

        # collect docx files to convert to
        docx_paths = []
        for pdf_path in self.pdf_paths:
            base_name = os.path.basename(pdf_path)
            name, ext = os.path.splitext(base_name)
            docx_path = os.path.join(self.docx_folder, f'{name}.docx')
            docx_paths.append(docx_path)
        
        if any([os.path.exists(path) for path in docx_paths]) and \
            not messagebox.askokcancel(title='Existed target file', 
                message='Docx files with same target name are found under selected folder. '
                        'Do you want to continue and replace them?'):
            return
        
        # now, do the converting work
        num_succ, num_fail = 0, 0
        for pdf_path, docx_path in zip(self.pdf_paths, docx_paths):
            cv = Converter(pdf_path)
            try:
                cv.convert(docx_path)
            except Exception as e:
                print(e)
                num_fail += 1
            else:
                num_succ += 1
            finally:
                cv.close()

        messagebox.showinfo(title='Convert Done!', 
            message=f'Successful ({num_succ}), Failed ({num_fail}).')
